"""Direct commands from the Insteom Modem to the device.

These messages represent commands sent to the device where the Insteon
Modem is the controller and the device is the responder in the
All-Link database.

Direct commands are sent to the device to trigger an All-Link group and/or
a state change on the device.
"""
